/*
	(C) 1995-99 AROS - The Amiga Research OS
	$Id: movewindowinfrontof.c,v 1.2 2009/02/22 21:34:28 jacadcaps Exp $
 
	Desc: Intuition function MoveWindowInFrontOf()
	Lang: english
*/
#include <proto/layers.h>
#include "intuition_intern.h"
#include "inputhandler_actions.h"
#include "mosmisc.h"

struct MoveWindowInFrontOfActionMsg
{
	struct IntuiActionMsg msg;
	struct Window *window;
	struct Window *behindwindow;
};

static VOID int_movewindowinfrontof(struct MoveWindowInFrontOfActionMsg *msg,
									struct IntuitionBase *IntuitionBase);

/*****************************************************************************
 
	NAME */
#include <proto/intuition.h>

AROS_LH2(void, MoveWindowInFrontOf,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window *, window, A0),
		 AROS_LHA(struct Window *, behindwindow, A1),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 80, Intuition)

/*  FUNCTION
	Arrange the relative depth of a window.
 
	INPUTS
	window - the window to reposition
	behindwindow - the window the other one will be brought in front of
 
	RESULT
	None.
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
	WindowToFront(), WindowToBack(), layers.library/MoveLayerInFrontOf()
 
	INTERNALS
	Uses layers.library/MoveLayerInFrontOf().
 
	HISTORY
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	struct MoveWindowInFrontOfActionMsg msg;

	SANITY_CHECK(window)
	SANITY_CHECK(behindwindow)

#ifdef __MORPHOS__
	if (IsDuringSelfResize(IntuitionBase)) return;
#endif

	msg.window = window;
	msg.behindwindow = behindwindow;
	DoASyncAction((APTR)int_movewindowinfrontof, &msg.msg, sizeof(msg), IntuitionBase);

	AROS_LIBFUNC_EXIT

} /* MoveWindowInFrontOf */


static VOID int_movewindowinfrontof(struct MoveWindowInFrontOfActionMsg *msg,
									struct IntuitionBase *IntuitionBase)
{
	struct Window *window = msg->window;
	struct Window *behindwindow = msg->behindwindow;
	struct Screen *screen = window->WScreen;
	struct Requester *req;
	struct Layer *layer = WLAYER(window);
	struct Layer *lay;
	BOOL movetoback = TRUE;

	LOCK_REFRESH(screen);

	for(lay = WLAYER(behindwindow); lay; lay = lay->back)
	{
		if (lay == layer)
		{
			movetoback = FALSE;
			break;
		}
	}

	MoveLayerInFrontOf(layer, WLAYER(behindwindow));

	if (BLAYER(window) && BLAYER(window) != layer)
	{
		MoveLayerInFrontOf(BLAYER(window), WLAYER(behindwindow));
	}

	for (req = window->FirstRequest; req; req = req->OlderRequest)
	{
		if (req->ReqLayer)
		{
			MoveLayerInFrontOf(req->ReqLayer,layer);
		}
	}

	#ifdef SKINS
	ArrangeChildren(window,IntuitionBase);
	#endif

	CheckLayers(screen, IntuitionBase);

	UNLOCK_REFRESH(screen);

	NotifyDepthArrangement(window, IntuitionBase);

	UpdateScreenPointer(screen,IntuitionBase);
}
