/*
	(C) 1995-99 AROS - The Amiga Research OS
	$Id: setwindowpointera.c,v 1.6 2012/03/26 18:06:41 jacadcaps Exp $
 
	Desc: Intuition function SetWindowPointerA()
	Lang: english
*/
#include <proto/graphics.h>
#include <proto/utility.h>
#include <intuition/pointerclass.h>
#include "intuition_intern.h"
#include "mosmisc.h"

#define NOBUTTONFOCUS (IBase->IControlPrefs.ic_Flags & ICF_NOMOUSEACTIONS)

struct SetWindowPointerMsg
{
	struct IntuiActionMsg msg;
	struct Window *win;
	struct TagItem *taglist;
};

void int_SetWindowPointer(struct SetWindowPointerMsg *msg,struct IntuitionBase *IntuitionBase);

/*****************************************************************************
 
	NAME */
#include <proto/intuition.h>

AROS_LH2(void, SetWindowPointerA,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window * , window , A0),
		 AROS_LHA(struct TagItem *, taglist, A1),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 136, Intuition)

/*  FUNCTION
 
	INPUTS
 
	RESULT
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	struct SetWindowPointerMsg msg;

	SANITY_CHECK(window);
	SANITY_CHECK(!(window->Flags & WFLG_WINDOWREFRESH));

	msg.win = window;
	msg.taglist = taglist;

#ifdef __MORPHOS__
	if (IsDuringSelfResize(IntuitionBase))
	{
		dprintf("Major bug: calling SetWindowPointer() during MUIM_Draw task %lx\n",(ULONG)FindTask(0));
		return;
	}
#endif

	DoSyncAction((APTR)int_SetWindowPointer,&msg.msg,IntuitionBase);

	AROS_LIBFUNC_EXIT
}

void int_SetWindowPointer(struct SetWindowPointerMsg *msg,struct IntuitionBase *IntuitionBase)
{
	struct Window *window = msg->win;
	struct TagItem *taglist = msg->taglist;

	DEBUG_SETPOINTER(dprintf("SetWindowPointer: window 0x%lx\n", window));

	if (window)
	{
		//ULONG lock;
		Object *pointer = (Object *)GetTagData(WA_Pointer, NULL, taglist);
		BOOL busy = (GetTagData(WA_BusyPointer, FALSE, taglist) != 0) ? TRUE : FALSE;

		#ifdef SKINS
		ULONG pointertype = GetTagData(WA_PointerType, POINTERTYPE_NORMAL, taglist);
		//if (pointertype != 0) dprintf("Pointertype %ld\n",pointertype);
		#endif

		#ifdef SKINS
		DEBUG_SETPOINTER(dprintf("SetWindowPointer: pointer 0x%lx busy %d type %d\n",
								 pointer, busy, pointertype));
		#endif

		DEBUG_SETPOINTER(dprintf("SetWindowPointer: pointer 0x%lx busy %d\n",
		                         pointer, busy));

		//if (!ILOCKCHECK(((struct IntuiActionMsg *)msg))) lock = LockIBase (0);

		DEBUG_SETPOINTER(dprintf("SetWindowPointer: old pointer 0x%lx busy %d\n",
		                         IW(window)->pointer, IW(window)->busy));

		if (IW(window)->pointer != pointer || IW(window)->busy != busy || IW(window)->pointertype != pointertype)
		{
			Object *oldpointer = NULL;

#ifdef SKINS
			if ((GetTagData(WA_PointerDelay, FALSE, taglist)) && ((!(NOBUTTONFOCUS)) || ((NOBUTTONFOCUS) && (window == IntuitionBase->ActiveWindow))))
			{
				IW(window)->pointer_delay_pointertype = pointertype;
				IW(window)->pointer_delay_pointer = pointer;
				IW(window)->pointer_delay_busy = busy;
				IW(window)->pointer_delay = 5;
			} else {
				if (IW(window)->free_pointer) oldpointer = IW(window)->pointer;
				IW(window)->pointertype = pointertype;
				IW(window)->pointer = pointer;
				IW(window)->busy = busy;
				IW(window)->free_pointer = FALSE;
				IW(window)->pointer_delay = 0;
				window->Pointer = NULL;
			}

			UpdateScreenPointer(window->WScreen,IntuitionBase);

#else
			if (IW(window)->free_pointer)
				oldpointer = IW(window)->pointer;

			IW(window)->pointer = pointer;
			IW(window)->busy = busy;
			IW(window)->free_pointer = FALSE;
			window->Pointer = NULL;

			if (window->Flags & WFLG_WINDOWACTIVE)
			{
				struct IntScreen *scr = IS(window->WScreen);

				if (GetTagData(WA_PointerDelay, FALSE, taglist) &&
						IntuitionBase->FirstScreen == &scr->Screen)
				{
					DEBUG_SETPOINTER(dprintf("SetWindowPointer: delay change\n"));
					IBase->PointerDelay = 5;
				}
				else
				{
					struct SharedPointer *shared_pointer;

					IBase->PointerDelay = 0;

					if (pointer == NULL)
						pointer = IBase->DefaultPointer;
					if (busy)
						pointer = IBase->BusyPointer;

					GetAttr(POINTERA_SharedPointer, pointer, (IPTR *)&shared_pointer);

					DEBUG_POINTER(dprintf("SetWindowPointer: scr 0x%lx pointer 0x%lx sprite 0x%lx\n",
					                      scr, pointer, shared_pointer->sprite));

					if (ChangeExtSpriteA(&scr->Screen.ViewPort,
					                     scr->Pointer->sprite, shared_pointer->sprite, NULL))
					{
						ObtainSharedPointer(shared_pointer, IntuitionBase);
						ReleaseSharedPointer(scr->Pointer, IntuitionBase);
						scr->Pointer = shared_pointer;
						if (window)
						{
							window->XOffset = shared_pointer->xoffset;
							window->YOffset = shared_pointer->yoffset;
						}
						SetPointerPos(scr->Screen.MouseX, scr->Screen.MouseY);
					}
					else
					{
						DEBUG_POINTER(dprintf("SetWindowPointer: can't set pointer.\n"));
					}
				}
			}
#endif

			DisposeObject(oldpointer);
		} else {
			IW(window)->pointer_delay = 0;
		}

		//if (!ILOCKCHECK(((struct IntuiActionMsg *)msg))) lock = LockIBase (0);
	}

} /* SetWindowPointerA */

struct SetWindowPointerTypeMsg
{
	struct IntuiActionMsg msg;
	struct Window *win;
	ULONG PointerType;
};

void int_SetWindowPointerTypeAsync(struct SetWindowPointerTypeMsg *msg, struct IntuitionBase *IntuitionBase)
{
	struct SetWindowPointerMsg nmsg;
	struct TagItem tags[] =
	{
		{WA_PointerType, msg->PointerType},
		{TAG_DONE, 0}
	};

	nmsg.win = msg->win;
	nmsg.taglist = tags;
	int_SetWindowPointer(&nmsg, IntuitionBase);
}

void SetWindowPointerTypeAsync(ULONG pointertype, struct Window *win, struct IntuitionBase *IntuitionBase)
{
	struct SetWindowPointerTypeMsg msg;

	msg.win = win;
	msg.PointerType = pointertype;

	DoASyncAction((APTR)int_SetWindowPointerTypeAsync, &msg.msg, sizeof (msg), IntuitionBase);
}

